import hashlib
import json
from pathlib import Path
import pickle
import os

from destine_analytics.paths import LOG_DIR

def get_available_expids() -> list[str]:
    """Get all available experiment IDs in the given log directory."""
    return [p.name for p in LOG_DIR.glob("*") if p.is_dir()]


def json_serialize(obj):
    if isinstance(obj, Path):
        return str(obj)
    raise TypeError(f"Object of type {type(obj).__name__} is not JSON serializable")


def get_cache_path(name, *args, **kwargs) -> Path:
    key = json.dumps([name, *args, kwargs], default=json_serialize)
    filename = hashlib.sha256(key.encode("utf-8")).hexdigest()
    directory = os.path.basename(__file__)
    return Path.home().resolve() / ".cache" / directory / filename


def cache_read(cache, name, /, *args, **kwargs):
    if cache is False:
        return
    cpath = get_cache_path(name, *args, **kwargs)
    if not cpath.exists():
        return None
    print(f"Reading '{name}' from cache...")
    with open(cpath, "rb") as fb:
        return pickle.load(fb)


def cache_write(cache, obj, name, /, *args, **kwargs):
    if cache is False:
        return
    print(f"Writing '{name}' to cache...")
    cpath = get_cache_path(name, *args, **kwargs)
    if not cpath.parent.exists():
        cpath.parent.mkdir(parents=True)
    with open(cpath, "wb") as fb:
        pickle.dump(obj, fb)
